"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initialiseEndpoints = exports.validateWrapperData = void 0;
const listenerHandler_1 = require("./listenerHandler");
const limiterHander_1 = require("./limiterHander");
const validateWrapperData = (data) => {
    if (!data.limiter) {
        data.endpoints.forEach((x) => {
            if (x.limiter) {
                /* Error thrown if limiter requested when mongo client was not provided */
                throw new Error('Cannot initialise listener without a provided mongoClient');
            }
        });
    }
};
exports.validateWrapperData = validateWrapperData;
const initialiseEndpoints = (data) => {
    const { endpoints, limiter, expressApp, validateRequest, getLimiterIdentifier, } = data;
    const limiterStore = {};
    const enabledList = [];
    endpoints.forEach((endpoint) => {
        var _a;
        const rateLimiterID = ((_a = endpoint.limiter) === null || _a === void 0 ? void 0 : _a.keyPrefix) || `${endpoint.name}rx`;
        let endpointLimiter;
        if (limiter) {
            endpointLimiter = limiterHander_1.initaliseRateLimiter({
                limiterStore,
                limiter,
                limiterOptions: endpoint.limiter,
                rateLimiterID,
            });
            if (endpointLimiter)
                limiterStore[rateLimiterID] = endpointLimiter;
        }
        const enabled = listenerHandler_1.runListener({
            expressApp: expressApp,
            endpoint,
            endpointLimiter,
            validateRequest,
            getLimiterIdentifier,
        });
        if (enabled)
            enabledList.push(endpoint.name);
    });
    return { limiterStore, enabledList };
};
exports.initialiseEndpoints = initialiseEndpoints;
