"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initaliseRateLimiter = void 0;
const rate_limiter_flexible_1 = require("rate-limiter-flexible");
const utils_1 = require("./utils");
const initaliseRateLimiter = ({ limiterStore, limiter, limiterOptions, rateLimiterID, }) => {
    if (!limiterOptions)
        return null;
    if (limiter.mongoClient) {
        return initialiseMongoLimiter({
            limiterStore,
            mongoClient: limiter.mongoClient,
            mongoOptions: limiter.limiterOptions,
            limiterOptions,
            rateLimiterID,
        });
    }
    throw new Error(`Limiter failed to be created for "${limiterOptions.keyPrefix}", no valid limiter found (mongo currently only valid option)`);
};
exports.initaliseRateLimiter = initaliseRateLimiter;
const initialiseMongoLimiter = ({ limiterStore, mongoClient, mongoOptions, limiterOptions, rateLimiterID, }) => {
    const { points, duration, keyPrefix } = limiterOptions;
    /* Duration accepts m/h/d shorthand, multiply by the appropriate value */
    const valDuration = utils_1.getTimeFromString(duration);
    if (limiterStore[rateLimiterID]) {
        throw new Error(`Duplicate keyprefix "${keyPrefix}" found, cannot create rate limiter`);
    }
    const rateLimiterOpts = Object.assign({ storeClient: mongoClient, points, duration: valDuration, keyPrefix }, mongoOptions);
    return new rate_limiter_flexible_1.RateLimiterMongo(rateLimiterOpts);
};
