"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runListener = void 0;
const messages_1 = require("./messages");
const validationHandler_1 = require("./validationHandler");
const handleRequest = ({ req, res, endpoint, endpointLimiter, validateRequest, getLimiterIdentifier, }) => {
    validationHandler_1.performValidation({
        req,
        endpoint,
        endpointLimiter,
        validateRequest,
        getLimiterIdentifier,
    }).then((validationResult) => {
        if (validationResult.success) {
            endpoint.execute(req, res, Object.assign(Object.assign({}, req.headers), validationResult.headers));
        }
        else {
            res
                .status(validationResult.status || 500)
                .send({ text: validationResult.error || messages_1.errorMessages.DEFAULT_ERROR });
        }
    });
};
const runListener = ({ expressApp, endpoint, endpointLimiter, validateRequest, getLimiterIdentifier, }) => {
    if (endpoint.disabled)
        return false;
    const getRequestOpts = (req, res) => {
        return {
            req,
            res,
            endpoint,
            endpointLimiter,
            validateRequest,
            getLimiterIdentifier,
        };
    };
    const listenerPath = endpoint.defaultPath
        ? `/${endpoint.defaultPath}/${endpoint.path}`
        : `/${endpoint.path}`;
    switch (endpoint.type.toUpperCase()) {
        case 'GET':
            expressApp.get(`${listenerPath}`, (req, res) => handleRequest(getRequestOpts(req, res)));
            break;
        case 'POST':
            expressApp.post(`${listenerPath}`, (req, res) => handleRequest(getRequestOpts(req, res)));
            break;
        case 'PUT':
            expressApp.get(`${listenerPath}`, (req, res) => handleRequest(getRequestOpts(req, res)));
            break;
        case 'DELETE':
            expressApp.delete(`${listenerPath}`, (req, res) => handleRequest(getRequestOpts(req, res)));
            break;
        default:
            throw new Error(`Invalid listener type found for ${endpoint.name}, '${endpoint.type}' is not valid`);
    }
    return true;
};
exports.runListener = runListener;
