"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.performValidation = void 0;
const messages_1 = require("./messages");
const checkRateLimiter = ({ endpointLimiter, identifier, }) => {
    return new Promise((resolve) => {
        endpointLimiter
            .consume(identifier)
            .then(() => {
            resolve(true);
        })
            .catch(() => {
            resolve(false);
        });
    });
};
const checkArrayContains = (currentFields, expectedFields) => {
    const missingFields = [];
    expectedFields.forEach((requiredField) => {
        if (!currentFields.includes(requiredField)) {
            missingFields.push(requiredField);
        }
    });
    if (missingFields.length > 0) {
        return {
            success: false,
            missingFields,
        };
    }
    return {
        success: true,
    };
};
const performValidation = ({ req, endpoint, endpointLimiter, validateRequest, getLimiterIdentifier, }) => new Promise((resolve) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    const { ip } = req;
    let identifier = ip;
    /* Allow for custom rate limit identifiers (e.g. get session key from header) */
    if (getLimiterIdentifier)
        identifier = getLimiterIdentifier(req);
    /* Incase custom function fails to get an identifier, fail all requests */
    if (!identifier) {
        resolve({ success: false, error: messages_1.errorMessages.BAD_IDENTIFIER });
        return;
    }
    /* If a limiter is present, confirm requests are not exceeded for identifier */
    if (endpointLimiter) {
        const rateLimiterPass = yield checkRateLimiter({
            endpointLimiter,
            identifier,
        });
        if (!rateLimiterPass) {
            resolve({
                success: false,
                status: 429,
                error: messages_1.errorMessages.RATE_LIMIT_REACHED,
            });
            return;
        }
    }
    /* If any headers are required for the endpoint, ensure they are present */
    if (endpoint.headers && endpoint.headers.length > 0) {
        const headerValidation = checkArrayContains(Object.keys(req.headers), endpoint.headers);
        if (!headerValidation.success) {
            resolve({
                success: false,
                status: 401,
                error: `One or more headers are required : ${(_a = headerValidation.missingFields) === null || _a === void 0 ? void 0 : _a.join('')}`,
            });
            return;
        }
    }
    /* If any fields are required in the body, ensure they are present */
    if (endpoint.body && endpoint.body.length > 0) {
        try {
            const bodyValidation = checkArrayContains(Object.keys(req.body), endpoint.body);
            if (!bodyValidation.success) {
                resolve({
                    success: false,
                    status: 401,
                    error: `One or more arguments are required in the body : ${(_b = bodyValidation.missingFields) === null || _b === void 0 ? void 0 : _b.join('')}`,
                });
                return;
            }
        }
        catch (e) {
            resolve({
                success: false,
                status: 400,
                error: 'Body must be in a valid JSON format',
            });
            return;
        }
    }
    /* Allow for custom validation methods */
    if (validateRequest) {
        const validationResult = yield validateRequest({
            headers: req.headers,
            body: req.body,
            endpoint,
        });
        if (validationResult.success === undefined) {
            throw new Error('Custom validation request must return an object or object promise containing a boolean success item (e.g. {success: true})');
        }
        resolve(validationResult);
        return;
    }
    resolve({ success: true });
}));
exports.performValidation = performValidation;
